import java.awt.*;
import java.applet.*;
public class FractalTree extends Applet
{
	public static final double change = Math.PI / 8; // The change in angle is based on this value.

	double[] random; // Array of random numbers, stored so the tree appears the same in every paint.
	int randomIndex; // how far are we into the random numbers?

	public int getWidth() // getWidth is defined for java 1.2, but not 1.1
	{
		return getSize().width;
	}

	public int getHeight() // same as getWidth.
	{
		return getSize().height;
	}

	public void init()
	{
	}

	public void start()  // Initialize the new tree's random values.
	{
		random = new double[64*1024];
		for (int i = 0; i < random.length; i++)
			random[i] = Math.random();
	}

	public double random() // get the next "random" value from the array
	{
		randomIndex ++;
		if (randomIndex >= random.length)
		{
			randomIndex = 0;
		}
		return random[randomIndex];
	}

	public void paint(Graphics g) // Paint the random tree.
	{	
		randomIndex = 0;	
		g.setColor(Color.blue);
		g.fillRect(0, 0, getWidth(), getHeight());
		drawBranch(g, getWidth()/2, getHeight(), getHeight() / 5, -Math.PI / 2, 0);
	}
	public void drawBranch(Graphics g, double x, double y, double length, double angle, int depth)
		     // A recursive function that draws part of the tree.
	{
		if (depth >= 16)
			return;
		double nx = x + (Math.cos(angle) * length); // tip of branch
		double ny = y + (Math.sin(angle) * length); // ...
		// Set the appropriat color (fades from red to green)
		g.setColor(new Color(255 - depth * 16, depth * 16, 0));
		// and draw the line of that color.
		g.drawLine((int)x, (int)y, (int)nx, (int)ny);

		int count = 0;
		do
		{
			double c1 = change * (.5 + random()); // a branch between change*.5 and change*1.5 radians different from current branch
			double c2 = change * (random() - .5); // a branch between -change*.5 and change*.5 radians different from current branch
			double c3 = change * (1 + random()); // a branch between change and change*2 radians different from current branch
			if (random() < .7) // 70% chance of right branch 1
			{
				drawBranch(g, nx, ny, length * 0.8, angle - c1 , depth + 1);
				count ++;
			}
			if (random() < .7) // 70% chance of left branch 1
			{
				drawBranch(g, nx, ny, length * 0.8, angle + c1, depth + 1);
				count ++;
			}
			if (random() < .2) // 20% chance of right branch 2
			{
				drawBranch(g, nx, ny, length * 0.8, angle - c2, depth + 1);
				count ++;
			}
			if (random() < .2) // 20% chance of left branch 2
			{
				drawBranch(g, nx, ny, length * 0.8, angle + c2, depth + 1);
				count ++;
			}
			if (random() < .1) // 10% chance of right branch 3 
			{
				drawBranch(g, nx, ny, length * 0.8, angle - c3, depth + 1);
				count ++;
			}
			if (random() < .1) // 10% chance of left branch 3
			{
				drawBranch(g, nx, ny, length * 0.8, angle + c3, depth + 1);
				count ++;
			}
		} while (count < 2 && depth < 4);  // If we are less then 4 up, insure atleast 2 branches.
	}
}
